/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSArrayController;

import dstep.appkit.NSObjectController;
import dstep.foundation.NSArray;
import dstep.foundation.NSIndexSet;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSPredicate;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSArrayController : NSObjectController
{
	mixin (ObjcWrap);

	void rearrangeObjects ()
	{
		return invokeObjcSelf!(void, "rearrangeObjects");
	}

	void setAutomaticallyRearrangesObjects (bool flag)
	{
		return invokeObjcSelf!(void, "setAutomaticallyRearrangesObjects:", bool)(flag);
	}

	bool automaticallyRearrangesObjects ()
	{
		return invokeObjcSelf!(bool, "automaticallyRearrangesObjects");
	}

	NSArray automaticRearrangementKeyPaths ()
	{
		return invokeObjcSelf!(NSArray, "automaticRearrangementKeyPaths");
	}

	void didChangeArrangementCriteria ()
	{
		return invokeObjcSelf!(void, "didChangeArrangementCriteria");
	}

	void setSortDescriptors (NSArray sortDescriptors)
	{
		return invokeObjcSelf!(void, "setSortDescriptors:", NSArray)(sortDescriptors);
	}

	NSArray sortDescriptors ()
	{
		return invokeObjcSelf!(NSArray, "sortDescriptors");
	}

	void setFilterPredicate (NSPredicate filterPredicate)
	{
		return invokeObjcSelf!(void, "setFilterPredicate:", NSPredicate)(filterPredicate);
	}

	NSPredicate filterPredicate ()
	{
		return invokeObjcSelf!(NSPredicate, "filterPredicate");
	}

	void setClearsFilterPredicateOnInsertion (bool flag)
	{
		return invokeObjcSelf!(void, "setClearsFilterPredicateOnInsertion:", bool)(flag);
	}

	bool clearsFilterPredicateOnInsertion ()
	{
		return invokeObjcSelf!(bool, "clearsFilterPredicateOnInsertion");
	}

	NSArray arrangeObjects (NSArray objects)
	{
		return invokeObjcSelf!(NSArray, "arrangeObjects:", NSArray)(objects);
	}

	Object arrangedObjects ()
	{
		return invokeObjcSelf!(Object, "arrangedObjects");
	}

	void setAvoidsEmptySelection (bool flag)
	{
		return invokeObjcSelf!(void, "setAvoidsEmptySelection:", bool)(flag);
	}

	bool avoidsEmptySelection ()
	{
		return invokeObjcSelf!(bool, "avoidsEmptySelection");
	}

	void setPreservesSelection (bool flag)
	{
		return invokeObjcSelf!(void, "setPreservesSelection:", bool)(flag);
	}

	bool preservesSelection ()
	{
		return invokeObjcSelf!(bool, "preservesSelection");
	}

	void setSelectsInsertedObjects (bool flag)
	{
		return invokeObjcSelf!(void, "setSelectsInsertedObjects:", bool)(flag);
	}

	bool selectsInsertedObjects ()
	{
		return invokeObjcSelf!(bool, "selectsInsertedObjects");
	}

	void setAlwaysUsesMultipleValuesMarker (bool flag)
	{
		return invokeObjcSelf!(void, "setAlwaysUsesMultipleValuesMarker:", bool)(flag);
	}

	bool alwaysUsesMultipleValuesMarker ()
	{
		return invokeObjcSelf!(bool, "alwaysUsesMultipleValuesMarker");
	}

	bool setSelectionIndexes (NSIndexSet indexes)
	{
		return invokeObjcSelf!(bool, "setSelectionIndexes:", NSIndexSet)(indexes);
	}

	NSIndexSet selectionIndexes ()
	{
		return invokeObjcSelf!(NSIndexSet, "selectionIndexes");
	}

	bool setSelectionIndex (NSUInteger index)
	{
		return invokeObjcSelf!(bool, "setSelectionIndex:", NSUInteger)(index);
	}

	NSUInteger selectionIndex ()
	{
		return invokeObjcSelf!(NSUInteger, "selectionIndex");
	}

	bool addSelectionIndexes (NSIndexSet indexes)
	{
		return invokeObjcSelf!(bool, "addSelectionIndexes:", NSIndexSet)(indexes);
	}

	bool removeSelectionIndexes (NSIndexSet indexes)
	{
		return invokeObjcSelf!(bool, "removeSelectionIndexes:", NSIndexSet)(indexes);
	}

	bool setSelectedObjects (NSArray objects)
	{
		return invokeObjcSelf!(bool, "setSelectedObjects:", NSArray)(objects);
	}

	NSArray selectedObjects ()
	{
		return invokeObjcSelf!(NSArray, "selectedObjects");
	}

	bool addSelectedObjects (NSArray objects)
	{
		return invokeObjcSelf!(bool, "addSelectedObjects:", NSArray)(objects);
	}

	bool removeSelectedObjects (NSArray objects)
	{
		return invokeObjcSelf!(bool, "removeSelectedObjects:", NSArray)(objects);
	}

	void add (Object sender)
	{
		return invokeObjcSelf!(void, "add:", Object)(sender);
	}

	void remove (Object sender)
	{
		return invokeObjcSelf!(void, "remove:", Object)(sender);
	}

	void insert (Object sender)
	{
		return invokeObjcSelf!(void, "insert:", Object)(sender);
	}

	bool canInsert ()
	{
		return invokeObjcSelf!(bool, "canInsert");
	}

	void selectNext (Object sender)
	{
		return invokeObjcSelf!(void, "selectNext:", Object)(sender);
	}

	void selectPrevious (Object sender)
	{
		return invokeObjcSelf!(void, "selectPrevious:", Object)(sender);
	}

	bool canSelectNext ()
	{
		return invokeObjcSelf!(bool, "canSelectNext");
	}

	bool canSelectPrevious ()
	{
		return invokeObjcSelf!(bool, "canSelectPrevious");
	}

	void addObject (Object object)
	{
		return invokeObjcSelf!(void, "addObject:", Object)(object);
	}

	void addObjects (NSArray objects)
	{
		return invokeObjcSelf!(void, "addObjects:", NSArray)(objects);
	}

	void insertObject (Object object, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertObject:atArrangedObjectIndex:", Object, NSUInteger)(object, index);
	}

	void insertObjects (NSArray objects, NSIndexSet indexes)
	{
		return invokeObjcSelf!(void, "insertObjects:atArrangedObjectIndexes:", NSArray, NSIndexSet)(objects, indexes);
	}

	void removeObjectAtArrangedObjectIndex (NSUInteger index)
	{
		return invokeObjcSelf!(void, "removeObjectAtArrangedObjectIndex:", NSUInteger)(index);
	}

	void removeObjectsAtArrangedObjectIndexes (NSIndexSet indexes)
	{
		return invokeObjcSelf!(void, "removeObjectsAtArrangedObjectIndexes:", NSIndexSet)(indexes);
	}

	void removeObject (Object object)
	{
		return invokeObjcSelf!(void, "removeObject:", Object)(object);
	}

	void removeObjects (NSArray objects)
	{
		return invokeObjcSelf!(void, "removeObjects:", NSArray)(objects);
	}
}

